import time
import asyncio

import pandas as pd


from brightml.features import p, d, get_features
from brightml.utils import save_sample
from brightml.brightness import BrightnessManager
from brightml.pipeline import get_classifier_pipeline
from brightml.file_watcher import watch_fs

try:
    # ensure_future() has been introduced in Python 3.4.4
    from asyncio import ensure_future
except ImportError:
    from asyncio import async as ensure_future


class Brightml():
    def __init__(self, display):
        self.old_value = None
        self.bm = BrightnessManager()
        self.display = display
        self.pipeline_clf = get_classifier_pipeline(None)

    def adjust_brightness(self):
        if self.pipeline_clf is not None:
            features = get_features()
            new_value = (features["ambient_light"],
                         features["display_window_name"], features["datetime_hour"])
            if self.old_value != new_value:
                #print("NEW STATE", features)
                if self.pipeline_clf:
                    try:
                        #t1 = time.time()
                        data = pd.DataFrame([features])
                        pipeline, clf = self.pipeline_clf
                        data = pipeline.transform(data)
                        # y is nans in this case
                        X, _ = data[:, :-1], data[:, -1]
                        pred = int(100 * clf.predict(X)[0])
                        tmpl = "new_brightness={}% app={}"
                        print(tmpl.format(pred, features["display_window_class"]))
                        self.bm.set_by_percentage(pred)
                    except ValueError as e:
                        print(e)
                        return
                else:
                    print("PREDICTING!")
                self.old_value = new_value

    async def retrain(self):
        await asyncio.sleep(0.7)
        features = get_features()
        features["new_brightness"] = self.bm.get_percentage()
        save_sample(features)
        print("RETRAINING")
        self.pipeline_clf = get_classifier_pipeline(None)
        print("RETRAINING DONE")
        self.adjust_brightness()


async def whereami():
    p.predict()
    await asyncio.sleep(30)
    ensure_future(whereami())


def main():
    bml = Brightml(d)
    from threading import Thread
    t = Thread(target=d.main_thread_fn, args=[bml.adjust_brightness])
    t.start()
    loop = asyncio.get_event_loop()
    if p is not None:
        loop.create_task(whereami())
    loop.run_until_complete(watch_fs(bml))
    t.join()


if __name__ == "__main__":
    main()



# todo: chargedness of battery

import time
from datetime import datetime
import numpy as np

try:
    from whereami.predict import Predicter
    p = Predicter()
except ImportError:
    p = None

from brightml.xdisplay import Display
d = Display()


DATE_FORMAT = '%Y-%m-%dT%H:%M:%S%z'


def get_ambient_light():
    try:
        # please, if your ambient light sensor doesn't work, post the path in the issues on github
        with open("/sys/devices/platform/applesmc.768/light") as f:
            return int(f.read()[1:-1].split(",")[0])
    except:
        return np.nan


def get_time_features():
    now = datetime.strptime(time.strftime(DATE_FORMAT, time.localtime()), DATE_FORMAT)
    data = {
        "datetime_hour": now.hour,
        "datetime_timezone": str(now.tzinfo),
        "datetime_date": str(now.date()),
        "datetime_full": str(now)
    }
    return data


def get_features():
    data = {
        "ambient_light": get_ambient_light(),
        "whereami": p.predicted_value if p else None
    }
    data.update(get_time_features())
    data.update(d.last_value)
    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import codecs

from setuptools import setup, find_packages

try:
    # Python 3
    from os import dirname
except ImportError:
    # Python 2
    from os.path import dirname

here = os.path.abspath(dirname(__file__))
with codecs.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist bdist_wheel upload")
    sys.exit()

required = [
    "zmq",
    "scipy",
    "numpy",
    "pandas",
    "scikit-learn",
    "python-xlib",
    "Pillow",
    "simplejson",
    "pyinotify"
]

extras_require = {
    'whereami': ['whereami']
}

# poor man's version parser
with open("brightml/__init__.py") as f:
    for line in f:
        version = line.strip().split()[-1].strip('"')
        break

setup(
    name='brightml',
    version=version,
    description='Machine Learned Auto brightness',
    long_description=long_description,
    author='Pascal van Kooten',
    author_email='kootenpv@gmail.com',
    url='https://github.com/kootenpv/brightml',
    packages=find_packages(),
    install_requires=required,
    license='MIT',
    entry_points={
        'console_scripts': ['brightml = brightml.__main__:main']
    },
    extras_require=extras_require,
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'

    ],
)
